﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Data.SQLite;
using System.Data.SqlClient;
using System.Data;

namespace AjpdSoftGestiónSeguidoresTwitter
{
    class BDSQLite
    {
        public SQLiteConnection conexionSQLite;

        //crear la BD SQLite y las tablas para guardar datos de Twitter
        public void crearBDSQLite (string fichero, string contrasena, ref string resultado)
        {     
            try
            {
                //creamos la BD SQLite
                SQLiteConnection.CreateFile(fichero);
                if (contrasena != "")
                {
                    //establecemos la contraseña
                    conexionSQLite.ChangePassword(contrasena);
                }
                //si existe el fichero de BD creamos las tablas
                if (System.IO.File.Exists(fichero))
                {
                    //conectamos con la BD SQLite para crear las tablas
                    string resultadoConexion = "";
                    if (conectarBDSQLite(fichero, contrasena, ref resultadoConexion))
                    {
                        //crear las tablas de SQLite para guardar datos de Twitter
                        //tabla seguidores
                        string consultaSQL = "create table seguidores (" +
                            "id long not null primary key, fecha datetime)";
                        SQLiteCommand comandoSQL =
                            new SQLiteCommand(consultaSQL, conexionSQLite);
                        comandoSQL.CommandType = CommandType.Text;
                        comandoSQL.ExecuteNonQuery();
                        //tabla amigos
                        consultaSQL = "create table amigos (" +
                            "id long not null primary key, fecha datetime)";
                        comandoSQL =
                            new SQLiteCommand(consultaSQL, conexionSQLite);
                        comandoSQL.CommandType = CommandType.Text;
                        comandoSQL.ExecuteNonQuery();
                        //tabla mensajes directos
                        consultaSQL = "create table mensaje (" +
                            "id not null primary key, idusuario long)";
                        comandoSQL =
                            new SQLiteCommand(consultaSQL, conexionSQLite);
                        comandoSQL.CommandType = CommandType.Text;
                        comandoSQL.ExecuteNonQuery();

                        resultado = System.DateTime.Now + " " +
                            "BD SQLite creada correctamente.";
                    }
                }
            }
            catch (Exception error)
            {
                resultado = System.DateTime.Now + " " +
                    "Error al abrir la BD SQLite: " + error.Message;
            }
        }



        //conectar con la base de datos SQLite
        public bool conectarBDSQLite (string fichero, 
            string contrasena, ref string resultado)
        {
            //si existe la BD SQLite
            if (System.IO.File.Exists(fichero))
            {
                string stringConexionSQLite =
                    String.Format("Data Source={0}", fichero);
                try
                {
                    conexionSQLite = new SQLiteConnection(stringConexionSQLite);
                    if (contrasena != "")
                        conexionSQLite.SetPassword(contrasena);
                    conexionSQLite.Open();
                    resultado = System.DateTime.Now + " " +
                        "Conectado a la BD SQLite correctamente";
                    return true;
                }
                catch (Exception error)
                {
                    resultado = System.DateTime.Now + " " +
                        "Error al abrir BD SQLite: " + error.Message;
                    return false;
                }
            }
            else
            {
                resultado = System.DateTime.Now + " " +
                    "Error al abrir BD SQLite: no existe el fichero " +
                    fichero;
                return false;
            }
        }


        //comprobar si existe seguidor en la bd SQLite (por el ID de Twitter)
        public bool existeSeguidor(long idSeguidor, ref string resultado)
        {
            string consultaSQL =
                "select count(*) from seguidores where id = " +
                Convert.ToString(idSeguidor);
            SQLiteCommand comandoSQL =
                new SQLiteCommand(consultaSQL, conexionSQLite);
            comandoSQL.CommandType = CommandType.Text;
            try
            {
                long numRegistros = (long)comandoSQL.ExecuteScalar();
                comandoSQL.Dispose();
                resultado = System.DateTime.Now + " " +
                    "Ejecutada consulta SQL en SQLite para existe seguidor: " +
                        Convert.ToString(numRegistros);
                return (numRegistros > 0);
            }
            catch (Exception error)
            {
                resultado = System.DateTime.Now + " " +
                    "Error SQLite existe seguidor: " + error.Message;
                return false;
            }
        }


        //insertar nuevo seguidor en BD SQLite
        public void insertarSeguidor(long idSeguidor, ref string resultado)
        {
            string consultaSQL =
                "insert into seguidores (id, fecha) values (@idSeguidor,  @fecha)";
            try
            {
                SQLiteParameter parametroFecha = new SQLiteParameter();
                parametroFecha.ParameterName = "@fecha";
                parametroFecha.DbType = DbType.DateTime;
                parametroFecha.Value = System.DateTime.Now;
                SQLiteParameter parametroidSeguidor = new SQLiteParameter();
                parametroidSeguidor.ParameterName = "@idSeguidor";
                parametroidSeguidor.DbType = DbType.String;
                parametroidSeguidor.Value = Convert.ToString(idSeguidor);
                SQLiteCommand comandoSQL =
                    new SQLiteCommand(consultaSQL, conexionSQLite);
                comandoSQL.CommandType = CommandType.Text;
                comandoSQL.Parameters.Add(parametroFecha);
                comandoSQL.Parameters.Add(parametroidSeguidor);
                comandoSQL.ExecuteNonQuery();
                resultado = System.DateTime.Now + " " +
                    "Insertado nuevo seguidor en BD SQLite [" +
                    Convert.ToString(idSeguidor) + "]";
            }
            catch (Exception error)
            {
                resultado = System.DateTime.Now + " " +
                    "Error SQLite insertar seguidor: " +
                    error.Message;
            }
        }
    }
}
